/*
 * NeuralNetworkViewFrame.java
 *
 * Created on October 4, 2008, 12:58 PM
 */

package org.neuroph.easyneurons;

import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
import javax.swing.JPanel;
import org.jdesktop.application.Action;
import org.neuroph.core.NeuralNetwork;
import org.neuroph.core.learning.TrainingSet;
import org.neuroph.easyneurons.TrainingController;
import org.neuroph.easyneurons.dialog.HebbianTrainingDialog;
import org.neuroph.easyneurons.dialog.LmsTrainingDialog;
import org.neuroph.easyneurons.dialog.SetNetworkInputDialog;
import org.neuroph.easyneurons.view.JNeuralNetwork;
import org.neuroph.util.NeuralNetworkType;

/**
 * 
 * @author Zoran Sevarac <sevarac@gmail.com>
 */
public class NeuralNetworkViewFrame extends javax.swing.JInternalFrame
		implements Observer {
	private static final long serialVersionUID = 1L;
	
	private NeuralNetwork neuralNetwork;
	private Vector<TrainingSet> trainingSets;
	private JNeuralNetwork blockView = new JNeuralNetwork();
	private NeuralGraphRenderer neuralGraphRenderer;
	private JPanel graphView;
	private TrainingController controller;
	private boolean graphViewActive = false;
        private String filePath;

	public static final int BLOCK_VIEW = 1;
	public static final int GRAPH_VIEW = 2;

	/** Creates new form NeuralNetworkViewFrame */
	public NeuralNetworkViewFrame(NeuralNetwork neuralNetwork,
			Vector<TrainingSet> trainingSets) {
		this.neuralNetwork = neuralNetwork;
		this.trainingSets = trainingSets;
		this.controller = new TrainingController(this.neuralNetwork);
		blockView.setNetwork(neuralNetwork);
		neuralNetwork.addObserver(this);

		// za rekurzivne modele baca exception
		try {
			neuralGraphRenderer = new NeuralGraphRenderer(this.neuralNetwork);
			graphView = neuralGraphRenderer.getGraphPanel();
		} catch (Exception ex) {
		}

		initComponents();

		this.jScrollPane1.getViewport().add(blockView);

		this.setTitle(neuralNetwork.toString());

		Iterator<TrainingSet> iterator = trainingSets.iterator();
		while (iterator.hasNext()) {
			TrainingSet ts = iterator.next();
			this.trainingSetsComboBox.addItem(ts);
		}

	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                jScrollPane1 = new javax.swing.JScrollPane();
                jPanel1 = new javax.swing.JPanel();
                jLabel1 = new javax.swing.JLabel();
                trainingSetsComboBox = new javax.swing.JComboBox();
                trainButton = new javax.swing.JButton();
                setInputButton = new javax.swing.JButton();
                calculateButton = new javax.swing.JButton();
                helpButton = new javax.swing.JButton();
                randomizeButton = new javax.swing.JButton();
                resetButton = new javax.swing.JButton();
                switchViewButton = new javax.swing.JButton();
                refreshButton = new javax.swing.JButton();

                setClosable(true);
                setIconifiable(true);
                setMaximizable(true);
                org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(org.neuroph.easyneurons.EasyNeuronsApplication.class).getContext().getResourceMap(NeuralNetworkViewFrame.class);
                setTitle(resourceMap.getString("Form.title")); // NOI18N
                setAutoscrolls(true);
                setDoubleBuffered(true);
                setMaximumSize(null);
                setName("Form"); // NOI18N
                setNormalBounds(null);
                setPreferredSize(null);

                jScrollPane1.setName("jScrollPane1"); // NOI18N
                jScrollPane1.setPreferredSize(new java.awt.Dimension(700, 600));
                getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

                jPanel1.setMaximumSize(null);
                jPanel1.setMinimumSize(null);
                jPanel1.setName("jPanel1"); // NOI18N
                jPanel1.setPreferredSize(null);
                jPanel1.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

                jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
                jLabel1.setName("jLabel1"); // NOI18N
                jPanel1.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 4, 70, -1));

                trainingSetsComboBox.setName("trainingSetsComboBox"); // NOI18N
                trainingSetsComboBox.addFocusListener(new java.awt.event.FocusAdapter() {
                        public void focusGained(java.awt.event.FocusEvent evt) {
                                trainingSetsComboBoxFocusGained(evt);
                        }
                });
                jPanel1.add(trainingSetsComboBox, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 0, 140, 23));

                javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(org.neuroph.easyneurons.EasyNeuronsApplication.class).getContext().getActionMap(NeuralNetworkViewFrame.class, this);
                trainButton.setAction(actionMap.get("train")); // NOI18N
                trainButton.setText(resourceMap.getString("trainButton.text")); // NOI18N
                trainButton.setName("trainButton"); // NOI18N
                jPanel1.add(trainButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(210, 0, -1, -1));

                setInputButton.setAction(actionMap.get("setInput")); // NOI18N
                setInputButton.setName("setInputButton"); // NOI18N
                jPanel1.add(setInputButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 0, 100, -1));

                calculateButton.setAction(actionMap.get("calculate")); // NOI18N
                calculateButton.setText(resourceMap.getString("calculateButton.text")); // NOI18N
                calculateButton.setName("calculateButton"); // NOI18N
                jPanel1.add(calculateButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(380, 0, -1, -1));

                helpButton.setText(resourceMap.getString("helpButton.text")); // NOI18N
                helpButton.setEnabled(false);
                helpButton.setName("helpButton"); // NOI18N
                jPanel1.add(helpButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(810, 0, -1, -1));

                randomizeButton.setAction(actionMap.get("randomize")); // NOI18N
                randomizeButton.setText(resourceMap.getString("randomizeButton.text")); // NOI18N
                randomizeButton.setName("randomizeButton"); // NOI18N
                jPanel1.add(randomizeButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(480, 0, -1, -1));

                resetButton.setAction(actionMap.get("reset")); // NOI18N
                resetButton.setName("resetButton"); // NOI18N
                jPanel1.add(resetButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(570, 0, -1, -1));

                switchViewButton.setAction(actionMap.get("switchView")); // NOI18N
                switchViewButton.setText(resourceMap.getString("switchViewButton.text")); // NOI18N
                switchViewButton.setName("switchViewButton"); // NOI18N
                jPanel1.add(switchViewButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(640, 0, -1, -1));

                refreshButton.setAction(actionMap.get("refresh")); // NOI18N
                refreshButton.setText(resourceMap.getString("refreshButton.text")); // NOI18N
                refreshButton.setName("refreshButton"); // NOI18N
                jPanel1.add(refreshButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(730, 0, -1, -1));

                getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

                pack();
        }// </editor-fold>//GEN-END:initComponents

	private void trainingSetsComboBoxFocusGained(java.awt.event.FocusEvent evt) {// GEN-FIRST:event_trainingSetsComboBoxFocusGained
		this.trainingSetsComboBox.removeAllItems();
		Iterator<TrainingSet> iterator = trainingSets.iterator();
		while (iterator.hasNext()) {
			TrainingSet ts = iterator.next();
			this.trainingSetsComboBox.addItem(ts);
		}
	}// GEN-LAST:event_trainingSetsComboBoxFocusGained

	@Action
	public void train() {
		TrainingSet trainingSet = (TrainingSet) trainingSetsComboBox
				.getSelectedItem();
		controller.setTrainingSet(trainingSet);

		NeuralNetworkType nnetType = neuralNetwork.getNetworkType(); // kod tipa
																	// mreze

		switch (nnetType) {
		case ADALINE:
			showLmsTrainingDialog();
			break;
		case PERCEPTRON:
			showLmsTrainingDialog(); /* perceptronTraining(); */
			break;
		case MULTI_LAYER_PERCEPTRON:
			showLmsTrainingDialog();
			break;
		case RBF_NETWORK:
			showLmsTrainingDialog(); /* showRbfTrainingDialog */
			break;
		case HOPFIELD:
			controller.train();
			break;
		case KOHONEN: /* KohonenTrainDlg(); */
			break;
		case NEURO_FUZZY_REASONER:
			showLmsTrainingDialog();
			break;
		case SUPERVISED_HEBBIAN_NET:
			showHebbianTrainingDialog();
			break;

		default:
			controller.train();
			break;
		} // switch*/

	}

	private void showLmsTrainingDialog() {
		LmsTrainingDialog trainingDialog = new LmsTrainingDialog(null, true,
				this.controller);
		trainingDialog.setLocationRelativeTo(this);
		trainingDialog.setVisible(true);
	}

	private void showHebbianTrainingDialog() {
		HebbianTrainingDialog trainingDialog = new HebbianTrainingDialog(null,
				true, this.controller);
		trainingDialog.setLocationRelativeTo(this);
		trainingDialog.setVisible(true);
	}

	@Action
	public void calculateNetwork() {
		this.neuralNetwork.calculate();
	}

	@Action
	public void setInput() {
		SetNetworkInputDialog dialog = new SetNetworkInputDialog(null, true,
				this.controller);
		dialog.setLocationRelativeTo(this);
		dialog.setVisible(true);
	}

	@Action
	public void calculate() {
		controller.calculate();
	}

	public TrainingController getController() {
		return this.controller;
	}

	@Action
	public void randomize() {
		controller.randomize();
	}

	@Action
	public void reset() {
		controller.reset();
	}

	@Action
	public void switchView() {
		if (!graphViewActive) {
			this.getContentPane().remove(jScrollPane1);
			this.getContentPane().add(graphView);
			graphViewActive = true;
			switchViewButton.setText("Block View");
		} else {
			this.getContentPane().remove(graphView);
			this.getContentPane().add(jScrollPane1);
			graphViewActive = false;
			switchViewButton.setText("Graph View");
		}

		this.repaint();
	}

	public void switchToView(int view) {
		if (view == GRAPH_VIEW) {
			this.getContentPane().remove(jScrollPane1);
			this.getContentPane().add(graphView);
			graphViewActive = true;
			switchViewButton.setText("Block View");
		} else if (view == BLOCK_VIEW) {
			this.getContentPane().remove(graphView);
			this.getContentPane().add(jScrollPane1);
			graphViewActive = false;
			switchViewButton.setText("Graph View");
		}

		this.repaint();
	}

	public void update(Observable o, Object arg) {
		if (graphViewActive) {
			neuralGraphRenderer.update();
			graphView = neuralGraphRenderer.getGraphPanel();
			graphView.revalidate();
			graphView.repaint();
		} else {
			blockView.update();
		}
	}

	@Action
	public void refresh() {

		if (graphViewActive) {
			this.getContentPane().remove(graphView);
			graphView = null;
			neuralGraphRenderer.update();
			graphView = neuralGraphRenderer.getGraphPanel();
			this.getContentPane().add(graphView);
			graphView.revalidate();
			graphView.repaint();
		} else {
			blockView.update();
		}

	}

	public void updateTitle() {
		this.setTitle(neuralNetwork.toString());
	}
	
	public String getFilePath() {
		return filePath;
	}

	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

	public NeuralNetwork getNeuralNetwork() {
		return neuralNetwork;
	}
        
        
        
        

        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JButton calculateButton;
        private javax.swing.JButton helpButton;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JPanel jPanel1;
        private javax.swing.JScrollPane jScrollPane1;
        private javax.swing.JButton randomizeButton;
        private javax.swing.JButton refreshButton;
        private javax.swing.JButton resetButton;
        private javax.swing.JButton setInputButton;
        private javax.swing.JButton switchViewButton;
        private javax.swing.JButton trainButton;
        private javax.swing.JComboBox trainingSetsComboBox;
        // End of variables declaration//GEN-END:variables

}
